/*
  SPDX-FileCopyrightText: 2024 Laurent Montel <montel.org>

  SPDX-License-Identifier: GPL-2.0-or-later
*/

#pragma once
#include "pimcommon_export.h"
#include <QString>

namespace PimCommon
{
namespace VerifyNewVersionUtils
{
[[nodiscard]] PIMCOMMON_EXPORT bool canVerifyNewVersion();
};
}
